clear all
close all
clc

%  Simulation for Subsection V.C - Spectrum estimation after channel selection (figs 7a 7b)


MC=5e4;
K=1e3;
M=30;
theta=ones(M,1);
theta(1:3)=theta(1:3)-linspace(0.02,0.05,3)';
theta(6:8)=3;
NN=round(logspace(2,4,6));


B=zeros(1,length(NN));
 psml_SA_mse=zeros(1,length(NN));
 psml_2b_mse=zeros(1,length(NN) );
 ml_mse=zeros(1,length(NN));
 ml_x_mse=zeros(1,length(NN));
 psml_68_mse=zeros(1,length(NN));
 split_mse=zeros(1,length(NN));

    
 psml_SA_bias=zeros(1,length(NN));
 psml_2b_bias=zeros(1,length(NN));
 ml_bias=zeros(1,length(NN));
 ml_x_bias=zeros(1,length(NN));
 psml_68_bias=zeros(1,length(NN));
 split_bias=zeros(1,length(NN));

 t_sa=zeros(1,length(NN));
 t_68=zeros(1,length(NN));
 t_2b=zeros(1,length(NN));
 
for n=1:length(NN) 
    N=NN(n);
    disp(N)
    Ny=round(0.1*N);
    Nx=N-Ny;   
    parfor l=1:MC
        x=(mvnrnd(zeros(Nx,M),diag(theta)))';
        theta_ml_x=mean(x.^2,2);
        [~,m]=min(theta_ml_x);
        y=(mvnrnd(zeros(Ny,1),theta(m)))';
        theta_ml_y=mean(y.^2,2);
        theta_ml=theta_ml_x;
        theta_ml(m)=(Nx/N)*theta_ml_x(m)+(Ny/N)*theta_ml_y;

        CRB=2*diag(theta_ml.^2)/Nx;
        CRB(m,m)=CRB(m,m)*Nx/N;
        tic
        theta_2b_psml=theta_ml_x;
        theta_2b_psml(m)=inf;
        [~,m2]=min(theta_2b_psml);

        theta_2b_psml=CE2best_psml(N,Nx,theta_ml,m,m2);
        t_2bl(l)=toc;

        
        %-----------------SA-PSML

        tic
        theta_SA_psml=theta_ml-CRB*g_se_sa_68(Nx,M,theta_2b_psml,m,K);
        t_sal(l)=toc;
        tic
        theta_SA68_psml=saCE68_iterative(N,Nx,M,theta_ml,m,K);
        t_l68(l)=toc;
        %---------------------------

        e_split(l)=theta_ml_y-theta(m);
        e_ml_x(l)=theta_ml_x(m)-theta(m);
        e_SA_psml(l)=theta_SA_psml(m)-theta(m); 
        e_2b_psml(l)=theta_2b_psml(m)-theta(m);
        e_ml(l)=theta_ml(m)-theta(m);
        e_68(l)=theta_SA68_psml(m)-theta(m);
        %---------------------------
        Jm=PSFIM_CE(Nx,M,theta,m,10*M*K);
        Jy=zeros(M);
        Jy(m,m)=0.5*Ny*theta(m).^(-2);
        Jm_inv=inv(Jy+Jm);
        Bm(l)=Jm_inv(m,m);
    end
    B(n)=mean(Bm);
    t_sa(n)=mean(t_sal);
    t_68(n)=mean(t_l68);
    t_2b(n)=mean(t_2bl);
    
    
    psml_SA_bias(n)=mean(e_SA_psml);
    psml_SA_mse(n)=mean(e_SA_psml.^2);
    
    psml_68_bias(n)=mean(e_68);
    psml_68_mse(n)=mean(e_68.^2);
    
    psml_2b_bias(n)=mean(e_2b_psml);
    psml_2b_mse(n)=mean(e_2b_psml.^2);
    
    split_bias(n)=mean(e_split);
    split_mse(n)=mean(e_split.^2);
    
    ml_bias(n)=mean(e_ml);
    ml_mse(n)=mean(e_ml.^2);
    
    ml_x_bias(n)=mean(e_ml_x);
    ml_x_mse(n)=mean(e_ml_x.^2);

end

save(strcat('..\\LCMEAS_mcode\simulations\',num2str(rand(1)),'_MC=',num2str(MC),'_M=',num2str(M),'CE2stages_N.mat'))
%%
LW=1.3;
fontsize=12;
figure
loglog(NN,t_sa,'-xb',NN,t_68,'-or',NN,t_2b,'-.sg','LineWidth',LW)

ylabel('Mean run-time [sec]')
xlabel('N')
legend('$\hat{\theta}^{(SA-PSML)}$','$\hat{\theta}^{(SA-PSML(II))}$','$\hat{\theta}^{(2nd-best)}$')
set(legend,'Interpreter','LaTex')
grid on
 set(legend,'FontSize',12)
 set(gca,'fontsize',12)
 xlim([min(NN),max(NN)])

figure
 loglog(NN,psml_68_mse,'--or','LineWidth',LW)
 hold on
 loglog(NN,psml_2b_mse,'--sg','LineWidth',LW)
 hold on
 loglog(NN,split_mse,'--.','color',[0.9290, 0.6940, 0.1250],'LineWidth',LW)
 hold on
 loglog(NN,ml_x_mse,'--d','color',[0,0.75,0.75],'LineWidth',LW)
hold on
 loglog(NN,ml_mse,'--*k','LineWidth',LW)
 hold on
loglog(NN,B,'-o','color',[0.5, 0, 0.5],'LineWidth',LW)


ylabel('PSMSE')
xlabel('N')
legend('$\hat{\theta}^{(SA-PSML)}$','$\hat{\theta}^{(2b-PSML)}$','$\hat{\theta}_y^{(ML)}$','$\hat{\theta}_x^{(ML)}$','$\hat{\theta}^{(ML)}$','$\Psi-CRB$')
set(legend,'Interpreter','LaTex')
grid on
 set(legend,'FontSize',12)
 set(gca,'fontsize',12)
 xlim([min(NN),max(NN)])

 
figure

semilogx(NN,abs(psml_68_bias),'--or','LineWidth',LW)
 hold on
semilogx(NN,abs(psml_2b_bias),'--sg','LineWidth',LW)
 hold on
semilogx(NN,abs(split_bias),'--.','color',[0.9290, 0.6940, 0.1250],'LineWidth',LW)
 hold on
semilogx(NN,abs(ml_x_bias),'--d','color',[0, 0.75, 0.75],'LineWidth',LW)
hold on
semilogx(NN,abs(ml_bias),'--*k','LineWidth',LW)

ylabel('$\Psi$-bias','interpreter','latex')
xlabel('N')
legend('$\hat{\theta}^{(SA-PSML)}$','$\hat{\theta}^{(2nd-best)}$','$\hat{\theta}_y^{(ML)}$','$\hat{\theta}_x^{(ML)}$','$\hat{\theta}^{(ML)}$')

set(legend,'Interpreter','LaTex')
grid on
 set(legend,'FontSize',12)
  set(gca,'fontsize',12)
  xlim([min(NN),max(NN)])
