clear all
close all
clc


%  Simulation for Subsection V.A - Linear Gaussian model (figs 5a 5b 5c)


MC=5e4; % Num of Monte Carlo simulations
M=25;
H=eye(M);   

C=toeplitz(1./((1:M)).^2);

theta=ones(M,1);
theta(2:3)=theta(2:3)+linspace(0.01,0.02,2)';
theta(1)=1.05;
theta(M)=0;
A=((H'*(C\H))\H')*C^-1;    

KK=round(logspace(1,3,6));

% initializations

e_SA_psml=zeros(1,MC);
e_2b_psml=zeros(1,MC);
e_ml=zeros(1,MC);
e_68=zeros(1,MC);
e_ml_y=zeros(1,MC);
e_CS=zeros(1,MC);
Bm=zeros(1,MC);



psml_SA_mse=zeros(1,length(KK));
psml_2b_mse=zeros(1,length(KK) );
ml_mse=zeros(1,length(KK));
psml_68_mse=zeros(1,length(KK));
ml_y_mse=zeros(1,length(KK));
B=zeros(1,length(KK));

CS_mse=zeros(1,length(KK));
CS_bias=zeros(1,length(KK));


    
 psml_SA_bias=zeros(1,length(KK));
 psml_2b_bias=zeros(1,length(KK));
 ml_bias=zeros(1,length(KK));
 psml_68_bias=zeros(1,length(KK));
 ml_y_bias=zeros(1,length(KK));

 t_sa=zeros(1,length(KK));
 t_68=zeros(1,length(KK));
 t_2b=zeros(1,length(KK));
 t_2bl=zeros(1,MC);
 t_sal=zeros(1,MC);
 t_l68=zeros(1,MC);
 
 %-----------------------------------------
 
 
SN=num2str(rand(1)); % just to creat unique fule name for samving 
NN=[150,500,1200];  
for n=1:length(NN) 
    N=NN(n); 
    Nx=round(0.8*N);
    Ny=N-Nx;
    Jx=Nx*H.'*(C\H);
    Jy=Ny*H.'*(C\H);
    J=Jx+Jy;

    CRBx=inv(Jx);
    CRBy=inv(Jy);
    CRB=inv(J);

    for k=1:length(KK)   
        K=KK(k); 
        display(K)
        parfor l=1:MC
            wx=(mvnrnd(zeros(Nx,M),C))';
            x=repmat(H*theta,1,Nx)+wx;
            x_mean=mean(x,2);
            theta_ml_x=A*x_mean;
            [~,m]=max(theta_ml_x);
            %----
            wy=(mvnrnd(zeros(Ny,M),C))';
            y=repmat(H*theta,1,Ny)+wy;
            theta_ml_y=A*mean(y,2);
            theta_ml=(Nx/N)*theta_ml_x+(Ny/N)*theta_ml_y;

           %-----------------SA-PSML
            tic
            theta_SA68_psml=sa68_iterative(N,Jx,CRBx,CRB,theta_ml,m,K);
            t_l68(l)=toc;
            %-------------------------

            e_68(l)=theta_SA68_psml(m)-theta(m);
     

        end

        t_68(n,k)=mean(t_l68);
        psml_68_mse(n,k)=mean(e_68.^2);
        psml_68_bias(n,k)=mean(e_68);
    end
end
%%
save(strcat('..\LCMEAS_mcode\simulations\',num2str(SN),' MC=',num2str(MC),' M=',num2str(M),'SA_2b_psml_vs_K_2stages.mat'))
%%
LW=1.3;
fontsize=12;
figure
loglog(KK,t_68(1,:),'--or','LineWidth',LW)
hold on
loglog(KK,t_68(2,:),'--xb','LineWidth',LW)
hold on
loglog(KK,t_68(3,:),'--sk','LineWidth',LW)
% hold on
% loglog(KK,t_68(4,:),'--*g','LineWidth',LW)


ylabel('Mean run-time [sec]')
xlabel('K')
legend(strcat('$N=$',num2str(NN(1))),strcat('$N=$',num2str(NN(2))),strcat('$N=$',num2str(NN(3))));
%legend(strcat('$N=$',num2str(NN(1))),strcat('$N=$',num2str(NN(2))),strcat('$N=$',num2str(NN(3))),strcat('$N=$',num2str(NN(4))));
set(legend,'Interpreter','LaTex')
grid on
 set(legend,'FontSize',fontsize)
 set(gca,'fontsize',fontsize)
 xlim([min(KK),max(KK)])

figure
loglog(KK,psml_68_mse(1,:),'--or','LineWidth',LW)
hold on
loglog(KK,psml_68_mse(2,:),'--xb','LineWidth',LW)
hold on
loglog(KK,psml_68_mse(3,:),'--sk','LineWidth',LW)
% hold on
% loglog(KK,psml_68_mse(4,:),'--*g','LineWidth',LW)

ylabel('PSMSE')
xlabel('K')
legend(strcat('$N=$',num2str(NN(1))),strcat('$N=$',num2str(NN(2))),strcat('$N=$',num2str(NN(3))));
%legend(strcat('$N=$',num2str(NN(1))),strcat('$N=$',num2str(NN(2))),strcat('$N=$',num2str(NN(3))),strcat('$N=$',num2str(NN(4))));
set(legend,'Interpreter','LaTex')
grid on
 set(legend,'FontSize',fontsize)
 set(gca,'fontsize',fontsize)
 xlim([min(KK),max(KK)])
% ylim([5e-3,1e-1])

figure

semilogx(KK,abs(psml_68_bias(1,:)),'--or','LineWidth',LW)
hold on
semilogx(KK,abs(psml_68_bias(2,:)),'--xb','LineWidth',LW)
hold on
semilogx(KK,abs(psml_68_bias(3,:)),'--sk','LineWidth',LW)
% hold on
% semilogx(KK,abs(psml_68_bias(4,:)),'--*g','LineWidth',LW)



ylabel('$\Psi$-bias','interpreter','latex')
xlabel('K')
legend(strcat('$N=$',num2str(NN(1))),strcat('$N=$',num2str(NN(2))),strcat('$N=$',num2str(NN(3))));
%legend(strcat('$N=$',num2str(NN(1))),strcat('$N=$',num2str(NN(2))),strcat('$N=$',num2str(NN(3))),strcat('$N=$',num2str(NN(4))));
set(legend,'Interpreter','LaTex')
grid on
 set(legend,'FontSize',fontsize)
  set(gca,'fontsize',fontsize)
  xlim([min(KK),max(KK)])
