clear all
close all
clc

%  Simulation for Subsection V.A - Linear Gaussian model (figs 2a 2b)

MC=5e4; % Num of Monte Carlo simulations
M=25;
H=eye(M);   

C=toeplitz(1./((1:M)).^2);  % Covsriance martix of the observations

theta=ones(M,1);
theta(2:3)=theta(2:3)+linspace(0.01,0.02,2)';
theta(1)=1.05;
theta(M)=0;
K=1e3; % num of Monte Carlo for SA
A=((H'*(C\H))\H')*C^-1;   

NN=round(logspace(2,4,6));  

% initializations-------------------

e_SA_psml=zeros(1,MC);
e_2b_psml=zeros(1,MC);
e_ml=zeros(1,MC);
e_68=zeros(1,MC);
e_ml_y=zeros(1,MC);
e_CS=zeros(1,MC);
Bm=zeros(1,MC);

ml_x_mse=zeros(1,length(NN));
psml_2b_mse=zeros(1,length(NN) );
ml_mse=zeros(1,length(NN));
psml_68_mse=zeros(1,length(NN));
ml_y_mse=zeros(1,length(NN));
JS_mse=zeros(1,length(NN));
B=zeros(1,length(NN));

CS_mse=zeros(1,length(NN));
CS_bias=zeros(1,length(NN));
   
 ml_x_bias=zeros(1,length(NN));
 psml_2b_bias=zeros(1,length(NN));
 ml_bias=zeros(1,length(NN));
 psml_68_bias=zeros(1,length(NN));
 ml_y_bias=zeros(1,length(NN));
 JS_bias=zeros(1,length(NN));

 t_sa=zeros(1,length(NN));
 t_68=zeros(1,length(NN));
 t_2b=zeros(1,length(NN));
 t_2bl=zeros(1,MC);
 t_sal=zeros(1,MC);
 t_l68=zeros(1,MC);
 
 %-----------------------------------------
 
for n=1:length(NN)  % run over different values of N the total num of observations
    N=NN(n); 

    display(N)
    Nx=round(0.8*N);  % num of the 1st stage 
    Ny=N-Nx;          % num of the 2nd stage 
    
    Jx=Nx*H.'*(C\H);
    Jy=Ny*H.'*(C\H);
    J=Jx+Jy;
    
    CRBx=inv(Jx);
    CRBy=inv(Jy);
    CRB=inv(J);

    parfor l=1:MC
        % 1st stage observation
        wx=(mvnrnd(zeros(Nx,M),C))';
        x=repmat(H*theta,1,Nx)+wx;
        x_mean=mean(x,2);
        theta_ml_x=A*x_mean;
        [~,m]=max(theta_ml_x);
        
        %----
        wy=(mvnrnd(zeros(Ny,M),C))';
        y=repmat(H*theta,1,Ny)+wy;
        theta_ml_y=A*mean(y,2);
        theta_ml=(Nx/N)*theta_ml_x+(Ny/N)*theta_ml_y;
        theta_bar=mean(theta_ml_x);
        
        %-- Shrinkage estimator
        JS_estimator=(1-(M-2)/(theta_ml'*J*theta_ml))*theta_ml;
        %----- CS estimator
        theta_CS=RPB_estimator(theta_ml_x,theta_ml_y,CRBx,CRBy);       
        %-------------2nd best
        tic
        temp=theta_ml_x;
        temp(m)=-inf;
        [~,m2]=max(temp);
        theta_2b_psml= psml_2best(CRB,CRBx,theta_ml,m,m2);
        t_2bl(l)=toc;
  
       %-----------------SA-PSML  ( "68" is an  old reference where this was the eq. num in the paper)
        tic
        theta_SA68_psml=sa68_iterative(N,Jx,CRBx,CRB,theta_ml,m,K);
        t_l68(l)=toc;
        %-------------------------

        e_2b_psml(l)=theta_2b_psml(m)-theta(m);
        e_ml(l)=theta_ml(m)-theta(m);
        e_ml_y(l)=theta_ml_y(m)-theta(m);
        e_ml_x(l)=theta_ml_x(m)-theta(m); 
        e_JS(l)=JS_estimator(m)-theta(m);
        e_68(l)=theta_SA68_psml(m)-theta(m);
        e_CS(l)=theta_CS-theta(m);
        
        %-------PSFIM--------------------
        Jm=PSFIM_gaussian_sms(Jx,CRBx,theta,m,10*M*K);
        Jm_inv=inv(Jy+Jm);
        Bm(l)=Jm_inv(m,m);
        
    end
    B(n)=mean(Bm);
    t_68(n)=mean(t_l68);
    t_2b(n)=mean(t_2bl);
    
    psml_68_mse(n)=mean(e_68.^2);
    psml_68_bias(n)=mean(e_68);
    
    psml_2b_mse(n)=mean(e_2b_psml.^2);
    psml_2b_bias(n)=mean(e_2b_psml);
 
    ml_y_mse(n)=mean(e_ml_y.^2);
    ml_y_bias(n)=mean(e_ml_y);
    
    ml_x_mse(n)=mean(e_ml_x.^2);
    ml_x_bias(n)=mean(e_ml_x);
    
    JS_mse(n)=mean(e_JS.^2);
    JS_bias(n)=mean(e_JS);
  
    ml_mse(n)=mean(e_ml.^2);
    ml_bias(n)=mean(e_ml);
     
    CS_mse(n)=mean(e_CS.^2);
    CS_bias(n)=mean(e_CS);
end
%%
save(strcat('..\LCMEAS_mcode\simulations\',num2str(rand(1)),' MC=',num2str(MC),' M=',num2str(M),'SA_2b_psml_vs_N_2stages.mat'))
%%
%-------plots-------
LW=1.3;
fontsize=12;
%-----run time------
% figure
% loglog(NN,t_sa,'--xb',NN,t_68,'--or',NN,t_2b,'--sg','LineWidth',LW)
% 
% ylabel('Mean run-time [sec]')
% xlabel('N')
% legend('$\hat{\theta}^{d-SA-PSML}$','$\hat{\theta}^{id-SA-PSML}$','$\hat{\theta}^{2B-PSML}$')
% set(legend,'Interpreter','LaTex')
% grid on
%  set(legend,'FontSize',fontsize)
%  set(gca,'fontsize',fontsize)
%  xlim([min(NN),max(NN)])

%------PSMSE fig-------
figure
loglog(NN,psml_68_mse,'--or','LineWidth',LW)
hold on
loglog(NN,psml_2b_mse,'--sg','LineWidth',LW)
hold on
loglog(NN,CS_mse,'--^m','LineWidth',LW)
hold on
loglog(NN,ml_y_mse,'--.','color',[0.9290, 0.6940, 0.1250],'LineWidth',LW)
hold on
loglog(NN,ml_x_mse,'--d','color',[0,0.75,0.75],'LineWidth',LW)
hold on
loglog(NN,JS_mse,'--xb','LineWidth',LW)
hold on

loglog(NN,ml_mse,'--*k','LineWidth',LW)
hold on
loglog(NN,B,'-o','color',[0.5, 0, 0.5],'LineWidth',LW)

ylabel('PSMSE')
xlabel('N')
legend('$\hat{\theta}^{(SA-PSML)}$','$\hat{\theta}^{(2b-PSML)}$','$\hat{\theta}^{(CS)}$','$\hat{\theta}_y^{(ML)}$','$\hat{\theta}_x^{(ML)}$','$\hat{\theta}^{(JS)}$','$\hat{\theta}^{(ML)}$','$\Psi-CRB$')
set(legend,'Interpreter','LaTex')
grid on
set(legend,'FontSize',fontsize)
set(gca,'fontsize',fontsize)
xlim([min(NN),max(NN)])

%----Psi-bias fig-------
figure

semilogx(NN,abs(psml_68_bias),'--or','LineWidth',LW)
hold on
semilogx(NN,abs(psml_2b_bias),'--sg','LineWidth',LW)
hold on
semilogx(NN,abs(CS_bias),'--^m','LineWidth',LW)
hold on
semilogx(NN,abs(ml_y_bias),'--.','color',[0.9290, 0.6940, 0.1250],'LineWidth',LW)
hold on
semilogx(NN,abs(ml_x_bias),'--d','color',[0, 0.75, 0.75],'LineWidth',LW)
hold on
semilogx(NN,abs(JS_bias),'--xb','LineWidth',LW)
hold on
semilogx(NN,abs(ml_bias),'--*k','LineWidth',LW)

ylabel('$\Psi$-bias','interpreter','latex')
xlabel('N')
legend('$\hat{\theta}^{(SA-PSML)}$','$\hat{\theta}^{(2b-PSML)}$','$\hat{\theta}^{(CS)}$','$\hat{\theta}_y^{(ML)}$','$\hat{\theta}_x^{(ML)}$','$\hat{\theta}^{(JS)}$','$\hat{\theta}^{(ML)}$')

set(legend,'Interpreter','LaTex')
grid on
set(legend,'FontSize',fontsize)
set(gca,'fontsize',fontsize)
xlim([min(NN),max(NN)])
