clear all
close all
clc

%  Simulation for Subsection V.B - Bernoulli model (figs 6a 6b)
% note: we refer to this simulation as "Binomial" since the sufficent
% statistics for estimation of theta is from Binomial distribution

%rng(2)
MC=5e4;
K=1e3;
M=25;
N=150;
theta=0.5*ones(M,1);
Delta=linspace(1e-3,0.15,7);
%----initializations---

B=zeros(1,length(Delta));
psml_SA_mse=zeros(1,length(Delta));
psml_2b_mse=zeros(1,length(Delta) );
ml_mse=zeros(1,length(Delta));
psml_68_mse=zeros(1,length(Delta));
split_mse=zeros(1,length(Delta));

    
psml_SA_bias=zeros(1,length(Delta));
psml_2b_bias=zeros(1,length(Delta));
ml_bias=zeros(1,length(Delta));
psml_68_bias=zeros(1,length(Delta));
split_bias=zeros(1,length(Delta));
t_sa=zeros(1,length(Delta));
t_68=zeros(1,length(Delta));
t_2b=zeros(1,length(Delta));

%------------------------------------
 Ny=round(0.25*N);
 Nx=N-Ny;
for n=1:length(Delta) 
    del=Delta(n)
    theta(1)=0.5+del;
    parfor l=1:MC
        x=binornd(Nx,theta,[M,1]);
        theta_ml_x=x/Nx;
        [~,m]=max(theta_ml_x);
        y=binornd(Ny,theta(m));
        theta_ml_y=y/Ny;
        theta_ml=theta_ml_x;
        theta_ml(m)=(Nx/N)*theta_ml_x(m)+(Ny/N)*theta_ml_y;
       
        crb_vec=(theta_ml.*(1-theta_ml))/Nx;
        crb_vec(crb_vec==0)=eps;
        if (sum(theta_ml.*(1-theta_ml)==0)~=0)
            disp('oi va voi')
        end
        CRB=diag(crb_vec);
        CRB(m,m)=CRB(m,m)*Nx/N;
    
        %--------2nd-Best PSML
        tic
        theta_2b_psml=theta_ml_x;
        theta_2b_psml(m)=-inf;
        [~,m2]=max(theta_2b_psml);
        theta_2b_psml=bin_psml_2best(N,Nx,theta_ml,m,m2) ;
        t_2bl(l)=toc;

        %-----------------SA-PSML
        tic
        theta_SA_psml=theta_ml-CRB*g_bin_sa_68(Nx,M,theta_2b_psml,m,K);
        t_sal(l)=toc;
        tic
        theta_SA68_psml=sa_bin_iterative(N,Nx,M,theta_ml,m,K);
        t_l68(l)=toc;
        %---------------------------

        e_split(l)=theta_ml_y-theta(m);
        e_ml_x(l)=theta_ml_x(m)-theta(m);
        e_SA_psml(l)=theta_SA_psml(m)-theta(m); 
        e_2b_psml(l)=theta_2b_psml(m)-theta(m);
        e_ml(l)=theta_ml(m)-theta(m);
        e_68(l)=theta_SA68_psml(m)-theta(m);
        %---------------------------
        Jm=PSFIM_binomial(Nx,M,theta,m,M*K);
        Jy=zeros(M);
        Jy(m,m)=Ny/(theta(m)*(1-theta(m)));
        Jm_inv=inv(Jy+Jm);
        Bm(l)=Jm_inv(m,m);
    end
    B(n)=mean(Bm);
    t_sa(n)=mean(t_sal);
    t_68(n)=mean(t_l68);
    t_2b(n)=mean(t_2bl);
    psml_SA_mse(n)=mean(e_SA_psml.^2);
    psml_68_mse(n)=mean(e_68.^2);
    psml_2b_mse(n)=mean(e_2b_psml.^2);
    split_mse(n)=mean(e_split.^2);
    ml_mse(n)=mean(e_ml.^2);
    ml_x_mse(n)=mean(e_ml_x.^2);

    
    psml_SA_bias(n)=mean(e_SA_psml);
    psml_2b_bias(n)=mean(e_2b_psml);
    ml_bias(n)=mean(e_ml);
    ml_x_bias(n)=mean(e_ml_x);
    psml_68_bias(n)=mean(e_68);
    split_bias(n)=mean(e_split);

end

save(strcat('..\\LCMEAS_mcode\simulations\',num2str(rand(1)),'_MC=',num2str(MC),'_N=',num2str(N),'_M=',num2str(M),'Bin_vs_delta.mat'))
%%
LW=1.3;
fontsize=12;
figure
loglog(Delta,t_sa,'-xb',Delta,t_68,'-or',Delta,t_2b,'-.sg','LineWidth',LW)

ylabel('Mean run-time [sec]')
xlabel('$\Delta$','interpreter','latex')
legend('$\hat{\theta}^{(SA-PSML)}$','$\hat{\theta}^{(SA-PSML(II))}$','$\hat{\theta}^{(2nd-best)}$')
set(legend,'Interpreter','LaTex')
grid on
 set(legend,'FontSize',12)
 set(gca,'fontsize',12)
 xlim([min(Delta),max(Delta)])

figure
plot(Delta,psml_68_mse,'--or','LineWidth',LW)
hold on
plot(Delta,psml_2b_mse,'--sg','LineWidth',LW)
hold on
plot(Delta,split_mse,'--.','color',[0.9290, 0.6940, 0.1250],'LineWidth',LW)
hold on
plot(Delta,ml_x_mse,'--d','color',[0,0.75,0.75],'LineWidth',LW)
hold on
plot(Delta,ml_mse,'--*k','LineWidth',LW)
hold on
plot(Delta,B,'-o','color',[0.5, 0, 0.5],'LineWidth',LW)


ylabel('PSMSE')
xlabel('$\Delta$','interpreter','latex')
legend('$\hat{\theta}^{(SA-PSML)}$','$\hat{\theta}^{(2b-PSML)}$','$\hat{\theta}_y^{(ML)}$','$\hat{\theta}_x^{(ML)}$','$\hat{\theta}^{(ML)}$','$\Psi-CRB$')
set(legend,'Interpreter','LaTex')
grid on
 set(legend,'FontSize',12)
 set(gca,'fontsize',12)
 xlim([min(Delta),max(Delta)])

 
figure
plot(Delta,abs(psml_68_bias),'--or','LineWidth',LW)
 hold on
plot(Delta,abs(psml_2b_bias),'--sg','LineWidth',LW)
 hold on
plot(Delta,abs(split_bias),'--.','color',[0.9290, 0.6940, 0.1250],'LineWidth',LW)
 hold on
plot(Delta,abs(ml_x_bias),'--d','color',[0, 0.75, 0.75],'LineWidth',LW)
hold on
plot(Delta,abs(ml_bias),'--*k','LineWidth',LW)

ylabel('$\Psi$-bias','interpreter','latex')
xlabel('$\Delta$','interpreter','latex')
legend('$\hat{\theta}^{(SA-PSML)}$','$\hat{\theta}^{(2b-PSML)}$','$\hat{\theta}_y^{(ML)}$','$\hat{\theta}_x^{(ML)}$','$\hat{\theta}^{(ML)}$')
set(legend,'Interpreter','LaTex')
grid on
set(legend,'FontSize',12)
set(gca,'fontsize',12)
xlim([min(Delta),max(Delta)])
